from installclass import BaseInstallClass
import rhpl
from rhpl.translate import N_,_
from constants import *
from flags import flags
import os
import iutil
import types
import re
try:
    import instnum
except ImportError:
    instnum = None

import logging
log = logging.getLogger("anaconda")

# custom installs are easy :-)
class InstallClass(BaseInstallClass):
    # name has underscore used for mnemonics, strip if you dont need it
    id = "xoslinux"
    name = N_("X/OS Linux")
    description = N_("The default installation of %s includes a set of "
                     "software applicable for general internet usage. "
                     "What additional tasks would you like your system "
                     "to include support for?") %(productName,)
    sortPriority = 10000
    allowExtraRepos = False
    if not productName.startswith("X/OS Linux"):
        hidden = 1

    taskMap = {'client'        : [(N_("Office"), ["office"]),
                                  (N_("Multimedia"), ["graphics", 
                                                      "sound-and-video"])],
               'server'        : [(N_("Software Development"), 
                                   ["development-libs", "development-tools",
                                    "gnome-software-development", 
                                    "x-software-development"],),
                                  (N_("Web server"), ["web-server"])],
               'workstation'   : [(N_("Software Development"), 
                                   ["development-libs", "development-tools",
                                    "gnome-software-development", 
                                    "x-software-development"],)],
               'vt'            : [(N_("Virtualization"), ["virtualization"])],
               'cluster'       : [(N_("Clustering"), ["clustering"])],
               'clusterstorage': [(N_("Storage Clustering"), 
                                   ["cluster-storage"])]
             }

    instkeyname = N_("Installation Number")

#    instkeydesc = N_("To install the full set of supported packages included "
#                    "in your subscription, please enter your Installation "
#                    "Number")
#    skipkeytext = N_("If you're unable to locate the Installation Number, "
#                    "consult http://www.redhat.com/apps/support/in.html.\n\n"
#                    "If you skip:\n"
#                    "* You may not get access to the full set of "
#                    "packages included in your subscription.\n"
#                    "* It may result in an unsupported/uncertified "
#                    "installation of Red Hat Enterprise Linux.\n"
#                    "* You will not get software and security updates "
#                    "for packages not included in your subscription.")
 
    instkeydesc = N_("To choose the installation type, enter a letter "
                    "or a combination of letters corresponding to the "
                    "package sets listed below or enter a valid Red Hat "
                    "Enterprise Linux installation number:\n"
                    "Client (C), Workstation (W), Server (S), "
                    "Virtualization (V), Cluster (U), Cluster Storage (G), "
                    "all package sets (A)")
    skipkeytext = N_("If you do not enter a valid installation number, "
                    "all package sets will be used for the installation.")
 

    def addRepoSet(self, repo, addcomps = True):
        if repo.lower() == "client":
            if addcomps:
                self.repopaths['clientgroups'] = "ClientGroups"
                log.info("Adding ClientGroups repo")
            self.repopaths['clientserver'] = "ClientServer"
            log.info("Adding ClientServer repo")
            self.repopaths['clientonly'] = "ClientOnly"
            log.info("Adding ClientOnly repo")
        elif repo.lower() == "workstation":
            if addcomps:
                self.repopaths['workstationgroups'] = "WorkstationGroups"
                log.info("Adding WorkstationGroups repo")
            self.repopaths['workstationserver'] = "WorkstationServer"
            log.info("Adding WorkstationServer repo")
            self.repopaths['workstationonly'] = "WorkstationOnly"
            log.info("Adding WorkstationOnly repo")
        elif repo.lower() == "server":
            if addcomps:
                self.repopaths['servergroups'] = "ServerGroups"
                log.info("Adding ServerGroups repo")
            self.repopaths['clientserver'] = "ClientServer"
            log.info("Adding ClientServer repo")
            self.repopaths['workstationserver'] = "WorkstationServer"
            log.info("Adding WorkstationServer repo")
        elif repo.lower() == "vt":
            if addcomps:
                self.repopaths['vtgroups'] = "VTGroups"
                log.info("Adding VTGroups repo")
            self.repopaths['vt'] = "VT"
            log.info("Adding VT repo")
        elif repo.lower() == "cluster":
            if addcomps:
                self.repopaths['clustergroups'] = "ClusterGroups"
                log.info("Adding ClusterGroups repo")
            self.repopaths['cluster'] = "Cluster"
            log.info("Adding Cluster repo")
        elif repo.lower() == "clusterstorage":
            if addcomps:
                self.repopaths['clusterstoragegroups'] = "ClusterStorageGroups"
                log.info("Adding ClusterStorageGroups repo")
            self.repopaths['clusterstorage'] = "ClusterStorage"
            log.info("Adding ClusterStorage repo")
        elif repo.lower() == "everything":
            self.repopaths['everythinggroups'] = "EverythingGroups"
            log.info("Adding EverythingGroups repo")
            # the Everything repo is empty, but we add it for completeness
            self.repopaths['everything'] = "Everything"
            log.info("Adding Everything repo")
            # recursively add the rest without groups     
            self.addRepoSet("Client", False)
            self.addRepoSet("Workstation", False)
            self.addRepoSet("Server", False)
            self.addRepoSet("VT", False)
            self.addRepoSet("Cluster", False)
            self.addRepoSet("ClusterStorage", False)

        if self.taskMap.has_key(repo.lower()):
            for task in self.taskMap[repo.lower()]:
                if task not in self.tasks:
                    self.tasks.append(task)
        self.tasks.sort()

    def updateYumConfig(self, rootPath):
        repoMap = { 'release'    : 'EverythingGroups',
                    'updates'    : 'EverythingGroups',
                    'release-clt': 'ClientGroups',
                    'updates-clt': 'ClientGroups',
                    'release-wrk': 'WorkstationGroups',
                    'updates-wrk': 'WorkstationGroups',
                    'release-srv': 'ServerGroups',
                    'updates-srv': 'ServerGroups',
                    'release-vrt': 'VTGroups',
                    'updates-vrt': 'VTGroups',
                    'release-cls': 'ClusterGroups',
                    'updates-cls': 'ClusterGroups',
                    'release-cst': 'ClusterStorageGroups',
                    'updates-cst': 'ClusterStorageGroups' }
        repos = self.repopaths.values()
        for relupd in ["release", "updates"]:
            yumConfig = "/etc/yum.repos.d/xoslinux-%s.repo" % (relupd,)
            if not os.path.isfile(rootPath + yumConfig):
                continue
            f = open(rootPath + yumConfig)
            lines = f.readlines()
            f.close()
            yumrepo = ""
            changed = False
            for i in range(0, len(lines)):
                m = re.match(r'\[(.*)\]\s*$', lines[i])
                if m:
                    yumrepo = m.group(1)
                m = re.match(r'enabled\s*=\s*([01])\s*$', lines[i])
                if m:
                    if repoMap.has_key(yumrepo):
                        enabled = m.group(1)
                        if repoMap[yumrepo] in repos and enabled != "1":
                            lines[i] = "enabled=1\n"
                            changed = True
                        elif repoMap[yumrepo] not in repos and enabled != "0":
                            lines[i] = "enabled=0\n"
                            changed = True
            if changed:
                f = open(rootPath + yumConfig, "w")
                f.writelines(lines)
                f.close()
        
    def setInstallData(self, anaconda):
	BaseInstallClass.setInstallData(self, anaconda)
        BaseInstallClass.setDefaultPartitioning(self, anaconda.id.partitions,
                                                CLEARPART_TYPE_LINUX)

    def setGroupSelection(self, anaconda):
        grps = anaconda.backend.getDefaultGroups(anaconda)
        map(lambda x: anaconda.backend.selectGroup(x), grps)

    def setSteps(self, dispatch):
	BaseInstallClass.setSteps(self, dispatch);
	dispatch.skipStep("partition")
	dispatch.skipStep("regkey",
		skip = (not flags.cmdline.has_key('askkey')))        

    # for rhel, we're putting the metadata under productpath
    def getPackagePaths(self, uri):
        rc = {}
        for (name, path) in self.repopaths.items():
            if not type(uri) == types.ListType:
                uri = [uri,]
            if not type(path) == types.ListType:
                path = [path,]

            lst = []
            for i in uri:
                for p in path:
                    lst.append("%s/%s" % (i, p))

            rc[name] = lst

        log.info("package paths is %s" %(rc,))
        return rc

    def handleRegKey(self, key, intf, interactive = True):

	# reset initialized repos

        self.tasks = []
        self.installkey = None
        self.repopaths = {}

        #RHEL# self.repopaths = { "base": "%s" %(productPath,) }
        #RHEL# self.tasks = self.taskMap[productPath.lower()]
        self.installkey = key

        def isValidXLKey(key):
            return re.match('\s*([CWSVUG]+[CWSVUG\s]*|A)\s*$', key.upper()) \
		and not re.match('[^CWS]*V[^CWS]*$', key.upper())

        try:
            inum = instnum.InstNum(key)
        except Exception, e:
            if True or not BETANAG: # disable hack keys for non-beta
                #RHEL# # make sure the log is consistent
                #RHEL# log.info("repopaths is %s" %(self.repopaths,))
                #RHEL# raise
                inum = None
                if not isValidXLKey(key):
                    self.addRepoSet("Everything")
                    raise Exception, e
            else:
                inum = None

        if inum is not None:
            #RHEL# # make sure the base products match
            #RHEL# if inum.get_product_string().lower() != productPath.lower():
            #RHEL#     raise ValueError, "Installation number incompatible with media"

            for name, path in inum.get_repos_dict().items():
                # virt is only supported on i386/x86_64.  so, let's nuke it
                # from our repo list on other arches unless you boot with
                # 'linux debug'
                if name.lower() == "virt" and ( \
                        rhpl.getArch() not in ("x86_64","i386")
                        and not flags.debug):
                    continue
                #RHEL# self.repopaths[name.lower()] = path
                #RHEL# log.info("Adding %s repo" % (name,))
                log.info("RHEL found name=%s path=%s" % (name, path))
                self.addRepoSet(path)

        elif isValidXLKey(key):

            key = key.upper()
            if re.search('A', key):
                # Everything selected
                self.addRepoSet("Everything")
            if re.search('[CW]', key):
                # Client and/or Workstation selected
                self.addRepoSet("Client")
            if re.search('W', key):
                # Workstation selected
                self.addRepoSet("Workstation")
            if re.search('[SUG]', key):
                # Server and/or Cluster and/or ClusterStorage selected
                self.addRepoSet("Server")
            if re.search('V', key):
                # VT selected
                self.addRepoSet("VT")
            if re.search('[UG]', key):
                # Cluster and/or ClusterStorage selected
                self.addRepoSet("Cluster")
            if re.search('G', key):
                # ClusterStorage selected
                self.addRepoSet("ClusterStorage")

        else:
            key = key.upper()
            # simple and stupid for now... if C is in the key, add Clustering
            # if V is in the key, add Virtualization. etc
            if key.find("C") != -1:
                self.repopaths["cluster"] = "Cluster"
                log.info("Adding Cluster option")
            if key.find("S") != -1:
                self.repopaths["clusterstorage"] = "ClusterStorage"
                log.info("Adding ClusterStorage option")
            if key.find("W") != -1:
                self.repopaths["workstation"] = "Workstation"
                log.info("Adding Workstation option")
            if key.find("V") != -1:
                self.repopaths["virt"] = "VT"
                log.info("Adding Virtualization option")

        #RHEL# for repo in self.repopaths.values():
        #RHEL#     if not self.taskMap.has_key(repo.lower()):
        #RHEL#         continue

        #RHEL#     for task in self.taskMap[repo.lower()]:
        #RHEL#         if task not in self.tasks:
        #RHEL#             self.tasks.append(task)
        #RHEL# self.tasks.sort()

        log.info("repopaths is %s" %(self.repopaths,))

    def __init__(self, expert):
	BaseInstallClass.__init__(self, expert)

	# build full repo list in case no key is provided
        self.tasks = []
        self.installkey = None
        self.repopaths = {}
        self.addRepoSet("Everything")

        # self.repopaths = { "base": "%s" %(productPath,) }

        # minimally set up tasks in case no key is provided
        # self.tasks = self.taskMap[productPath.lower()]

